Package: sets
Version: 1.0-22
Title: Sets, Generalized Sets, Customizable Sets and Intervals
Description: Data structures and basic operations for ordinary sets,
             generalizations such as fuzzy sets, multisets, and
             fuzzy multisets, customizable sets, and intervals.
Authors@R: c(person(given = "David", family = "Meyer", role = c("aut", "cre"),
                    email = "David.Meyer@R-project.org",
		    comment = c(ORCID = "0000-0002-5196-3048")),	    
             person(given = "Kurt", family = "Hornik", role = "aut",
	            email = "Kurt.Hornik@R-project.org",
                    comment = c(ORCID = "0000-0003-4198-9911")),
             person(given = "Christian", family = "Buchta", role = "ctb"))
Depends: R (>= 2.7.0)
Suggests: proxy
Imports: graphics,grDevices,stats,utils
License: GPL-2
NeedsCompilation: yes
Packaged: 2023-02-01 11:02:23 UTC; hornik
Author: David Meyer [aut, cre] (<https://orcid.org/0000-0002-5196-3048>),
  Kurt Hornik [aut] (<https://orcid.org/0000-0003-4198-9911>),
  Christian Buchta [ctb]
Maintainer: David Meyer <David.Meyer@R-project.org>
Repository: CRAN
Date/Publication: 2023-02-01 13:22:05 UTC
